--[[
Simple Wake-on-LAN utility for SqueezePlay

Author: Thomas Houlberg, thomas.houlberg@gmail.com

This script is free to distribute and use.

Function: Enables the SqueezePlay to send macgic packets to devices such as mediacenters, NAS systems, PC, etc that supports Wake-on-LAN.


--]]

local oo					= require("loop.simple")
local string				= require("string")
local pairs 				= pairs
local ipairs 				= ipairs
local table 				= table
local tostring	 			= tostring
local tonumber 				= tonumber
local Applet				= require("jive.Applet")
local Window				= require("jive.ui.Window")
local SimpleMenu			= require("jive.ui.SimpleMenu")
local Choice				= require("jive.ui.Choice")
local log					= require("jive.utils.log").addCategory("MyHome", jive.utils.log.DEBUG)
local Socket				= require("socket.core")
local Textinput				= require("jive.ui.Textinput")
local Textarea				= require('jive.ui.Textarea')
local Popup					= require("jive.ui.Popup")

local EVENT_SHOW			= jive.ui.EVENT_SHOW
local EVENT_HIDE			= jive.ui.EVENT_HIDE
local EVENT_CONSUME			= jive.ui.EVENT_CONSUME
local EVENT_REJECT			= jive.ui.EVENT_REJECT

module(...)
oo.class(_M, Applet)

function editWolEntry(self, index)
	local wincap = ""
	local isnew = false
	local wolentry = {}
	local menuitems = {}
	local lt1, lt2, lt3 = "", "", ""
	if index == nil then 
		isnew = true
		wincap = self:string("CREATE_WOL_ENTRY")
		wolentry = {name = "", mac = "000000000000"}
		lt1 = "1. "
		lt2 = "2. "
		lt3 = "3. "
	else
		wolentry = self:getSettings().wolEntries[index]
		wincap = self:string("EDIT_WOL_ENTRY")
	end
	local window = Window("window", wincap)
	table.insert(menuitems, {
							text		= lt1..tostring(self:string("DISPLAY_NAME")),
							callback	= function(evt, menuItem)
												changeDisplayName(self, wolentry)
											end
							})
	table.insert(menuitems, {
							text		= lt2..tostring(self:string("DEVICE_MAC_ADDR")),
							callback	= function(evt, menuItem)
												changeMacAddr(self, wolentry)
											end
							})
	if isnew then
		table.insert(menuitems, {
								text		= lt3..tostring(self:string("CREATE")),
								callback	= function(evt, menuItem)
													if (wolentry.name != "") and (wolentry.mac != "000000000000") then
														table.insert(self:getSettings().wolEntries, wolentry)
														self:storeSettings()
													end
													window:playSound("WINDOWSHOW")
													window:hide(Window.transitionPushLeft)
												end
							})

	else
		table.insert(menuitems, {
								text		= self:string("UPDATE"),
								callback	= function(evt, menuItem)
													if (wolentry.name != "") and (wolentry.mac != "000000000000") then
														self:getSettings().wolEntries[index] = wolentry
														self:storeSettings()
													end
													window:playSound("WINDOWSHOW")
													window:hide(Window.transitionPushLeft)
												end
							})
		
		table.insert(menuitems, {
								text		= self:string("DELETE"),
								callback	= function(evt, menuItem)
													self:getSettings().wolEntries[index] = nil
													self:storeSettings()
													window:playSound("WINDOWSHOW")
													window:hide(Window.transitionPushLeft)
												end
							})
	end
	
	local menu = SimpleMenu("menu", menuitems)
	window:addWidget(menu)
	window:show()

end


function addNewWolEntry(self)
	editWolEntry(self, nil)

end

 
function menusettings(self)
	local window = Window("window", self:string("WOL"))
	local menu = SimpleMenu("menu", {})

	window:addListener(EVENT_HIDE, function(evt, menuItem)
										window:removeWidget(menu)
										return EVENT_REJECT
									end
						)
						
	window:addListener(EVENT_SHOW, function(evt, menuItem)
										local wolentries = self:getSettings().wolEntries
										local items = {}
										table.insert(items, {
															text		= self:string("ADD_NEW"),
															callback	= function(evt, menuItem)
																			addNewWolEntry(self)
																			end
														})
										for i, wolentry in ipairs(wolentries) do
											table.insert(items, {
																text 		= wolentry.name,
																callback	= function(evt, menuItem)
																				editWolEntry(self, i)
																			end
																})
										end
										menu = SimpleMenu("menu", items)
										window:addWidget(menu)
										return EVENT_REJECT
									end
						)	
	window:show()
	
end


function changeDisplayName(self, wolentry)
	local window = Window("window", self:string("ENTER_DISPLAY_NANE"))
	
	local v = Textinput.textValue(wolentry.name, 1, 30)
	local name = Textinput("textinput",v,
				function(self1, value)
					wolentry.name = value.s
					window:playSound("WINDOWSHOW")
					window:hide(Window.transitionPushRight)
				end
			)
	window:addWidget(name)
	window:show()
	
end


function changeMacAddr(self, wolentry)
	local window = Window("window", self:string("ENTER_MAC_ADDRESS"))
	local mac = wolentry.mac
	local v = Textinput.hexValue(mac)
	local input = Textinput("textinput", v,
				function(_, value)
					wolentry.mac = value:getValue()
					window:playSound("WINDOWSHOW")
					window:hide(Window.transitionPushLeft)
				end)

	local help = Textarea("help", self:string("WOL_HELP"))

	window:addWidget(help)
	window:addWidget(input)

	window:show()
	return window
end

function toHex(str)
	local ret = ""
	local byteval = tonumber(str,16)
	if (byteval != 0) then
		ret = string.format("%c", byteval)
	else
		ret = "\0"
	end
	return ret
end


function sendWOL(self, macaddr)
	log:info("Sending Magicpacket to ", macaddr)
	local mp = string.format("%c%c%c%c%c%c", 255, 255, 255, 255, 255, 255) -- Leading "0xFFFFFFFFFFFF" for the magic packet
	local binmac = ""
	for mc = 1, 12, 2 do
		binmac = binmac .. toHex(macaddr:sub(mc, mc+1)) -- Convert the string based MAC address to a binary format
	end
	mp = mp .. string.rep(binmac, 16) -- Repeat the binary MAC 16 times and add to magic packet
	local wol = Socket.udp() -- Create UDP socket
	wol:setoption("broadcast", true) -- Allow broadcasts using UDP
	wol:sendto(mp, "255.255.255.255", 9) -- Broadcast the magic packed using UDP port 9 (Discard)
end


function menu(self)
	local window = Window("window", self:string("WOL"))
	local wolentries = self:getSettings().wolEntries
	local menuitems = {}
	if wolentries[1] != nil then
		for i, wolentry in ipairs(wolentries) do
			table.insert(menuitems, {
									text = wolentry.name,
									icon = Choice("choice",
													{" ", " "},
													function(self1, selectedIndex)
														sendWOL(self, wolentry.mac)
													end,
													1)
									})
		end
		local menu = SimpleMenu("menu", menuitems)
		window:addWidget(menu)
	else
		local hlptext = Textarea("textarea", self:string("NO_ENTRIES_HELP"))
		window:addWidget(hlptext)
	end
	window:show()
end




